#include <Arduino.h>
#include <stdio.h>
#include <iostream>
#include <vector>
#include <numeric>
#include <wire.h>

/*  
 Test the tft.print() viz embedded tft.write() function

 This sketch used font 2, 4, 7
 
 Make sure all the display driver and pin connections are correct by
 editing the User_Setup.h file in the TFT_eSPI library folder.

 Note that yield() or delay(0) must be called in long duration for/while
 loops to stop the ESP8266 watchdog triggering.

 #########################################################################
 ###### DON'T FORGET TO UPDATE THE User_Setup.h FILE IN THE LIBRARY ######
 #########################################################################
 */

// Graphics and font library for ST7735 driver chip
#include <TFT_eSPI.h>
#include <SPI.h>
#include <stdio.h>
#include <Adafruit_GPS.h>
#include <BLEDevice.h>
#include <BLEServer.h>
#include <BLEUtils.h>
#include <BLE2902.h>
#include "esp_log.h"
#include "driver/i2c.h"

#define configUSE_PREEMPTION 1
#define configUSEIDLEHOOK 0
#define configUSETICKHOOK 0
#define configCPUCLOCKHZ (( unsigned long) 1600000)
#define configMAXPRIORITIES ((unsigned portBASETYPE) 2)
#define configMINIMALSTACKSIZE ((unsigned short) 1024)
#define configTOTALHEAPSIZE ((size_t)(60*1024))

//int's for example sensor data
int s1 = 1;
int s2 = 100;
char s3[9];
int s4 = 69;
int s5 = 45;
int s6 = 10;

//setup for ble uart -------------------------------------------------------------------------------------------------------------------------------
BLEServer *pServer = NULL;
BLECharacteristic * pTxCharacteristic;
bool deviceConnected = false;
bool oldDeviceConnected = false;
uint8_t txValue = 0;

// See the following for generating UUIDs:
// https://www.uuidgenerator.net/

#define SERVICE_UUID           "6E400001-B5A3-F393-E0A9-E50E24DCCA9E" // UART service UUID
#define CHARACTERISTIC_UUID_RX "6E400002-B5A3-F393-E0A9-E50E24DCCA9E"
#define CHARACTERISTIC_UUID_TX "6E400003-B5A3-F393-E0A9-E50E24DCCA9E"


class MyServerCallbacks: public BLEServerCallbacks {
   void onConnect(BLEServer* pServer) {
     deviceConnected = true;
   };


   void onDisconnect(BLEServer* pServer) {
     deviceConnected = false;
   }
};


//this Callback is used to receive data from the central (phone) but you won't be sending data from phone.
class MyCallbacks: public BLECharacteristicCallbacks {
   void onWrite(BLECharacteristic *pCharacteristic) {
     std::string rxValue = pCharacteristic->getValue();


     if (rxValue.length() > 0) {
       Serial.println("*********");
       Serial.print("Received Value: ");
       for (int i = 0; i < rxValue.length(); i++)
         Serial.print(rxValue[i]);


       Serial.println();
       Serial.println("*********");
     }
   }
};


//----------------------------------------------------------------------------------------------

// init for GPS
float Total_Distance_Traveled = 0.0;
float previousLatAnchor = 0;
float previousLongAnchor = 0;
float currentLat;
float currentLong;
float current_Distance_Traveled = 0;
int currentMile = 1;
int CurrentSplitTime_minutes = 0;
int previousSplitTimeStart = 0;

char Current_elapsed_time[9];
int startHour;
int startMinute;
int startSecond;
int elapsedSeconds;
int elapsedMinutes;
int elapsedHours;

//----------------------------------------------------------------------------------------------

// init for LCD
TFT_eSPI tft = TFT_eSPI();  // Invoke library, pins defined in User_Setup.h

#define TFT_GREY 0x5AEB // New colour
int hr = 67;
float velo = 11.7;




// Calorie Data Initialization -------------------------------------------------------------------------------------------------------------
double weight = 90; // KG
double MET = 8;
double calories;


double caloriesBurned(int weight, int time, int MET){
  // time in seconds, weight in kg
    calories = ((time/3600) * 60 * MET * 3.5 * weight) / 200;
    return calories;
}




// GPS Initilization -------------------------------------------------------------------------------------------------------

#define GPSSerial Serial2
//SoftwareSerial mySerial(18, 19);
//Serial_M = uart_set_pin(UART_NUM_2, UART_PIN_NO_CHANGE, UART_PIN_NO_CHANGE, 18, 19);
// Set GPSECHO to 'false' to turn off echoing the GPS data to the Serial console
// Set to 'true' if you want to debug and listen to the raw GPS sentences
#define GPSECHO  true


Adafruit_GPS GPS(&Serial2);

//----------------------------------------------------------------------------------------------

// init for Accelerometer
const int MPU_addr=0x68;
int16_t AcX,AcY,AcZ,Tmp,GyX,GyY,GyZ;
 
int minVal=265;
int maxVal=402;
 
double x;
double y;
double z;


// init for HRM----------------------------------------------------------------------------------------------

int PulseSensorPurplePin0 = 34;        // Pulse Sensor PURPLE WIRE connected to ANALOG PIN 0
int PulseSensorPurplePin1 = 35;        // Pulse Sensor PURPLE WIRE connected to ANALOG PIN 1

int LED21 = 21;   //  The on-board Arduion LED

int Signal0;                // holds the incoming raw data. Signal value can range from 0-1024
int Signal1;

int Threshold = 2500;            // Determine which Signal to “count as a beat”, and which to ingore.
int i = 1;
int BPM0 = 60;
int lastBPM0;
int timeBetween0;
int iterTime0 = 0;
int BPMtime0;
int BPM1 = 60;
int lastBPM1;
int timeBetween1;
int iterTime1 = 0;
int BPMtime1;
int j = 0;
std::vector<int> HRvector;
int totalTime0;
int totalTime1;

int avBPM;
int avBPMtest = 60;
std::vector<int> test_vector;
int mean = 0;

int HR_denom = 0;
int HR_sum = 0;
int HR_stored_avg = 0;


float avg2(std::vector<int> const& v) {
    mean = accumulate(v.begin(), v.end(), 0.0) / v.size();
    return mean;
}



/*RTOS Begin -------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------*/

TaskHandle_t HRonSerialhandle;


//HRM RTOS ----------------------------------------------------------------------------------------------


  void HRonSerial(void *parameter){
    while(1){
        totalTime0 = millis();
        HRvector.clear();
        while(totalTime0 + 8000 > millis()){
            Signal0 = analogRead(PulseSensorPurplePin0);  // Read the PulseSensor’s value.
            if(Signal0 > Threshold){ // && (BPM0 > 30) && (BPM0 < 220)){
                lastBPM0 = BPM0;
                BPMtime0 = millis();
                timeBetween0 = BPMtime0 - iterTime0;
                BPM0 = (60000) / (1+timeBetween0);
                iterTime0 = iterTime0 + timeBetween0;
                if( (BPM0 > 30) && (BPM0 < 220)){
                  HRvector.push_back((BPM0));
                  delay(10);
                }
                delay(250);
            }
            }

        avBPM = avg2(HRvector);
        if (avBPM < 220 && avBPM > 30){
            Serial.print("Heartbeat ");
            Serial.print(i);
            Serial.print(", ");
            Serial.print(avBPM);
            Serial.println(" BPM");
            i++;
            HR_denom = i;
            HR_sum += avBPM;
            HR_stored_avg = HR_sum/HR_denom;
        }
        totalTime0 = 0;
        delay(20);
    }
 }


void GPSdata(void *parameter){
  while(1){
    char c = GPS.read();
    // if you want to debug, this is a good time to do it!
    if ((c) && (GPSECHO))
      Serial.write(c);




    // if a sentence is received, we can check the checksum, parse it...
    if (GPS.newNMEAreceived()) {
      // a tricky thing here is if we print the NMEA sentence, or data
      // we end up not listening and catching other sentences!
      // so be very wary if using OUTPUT_ALLDATA and trytng to print out data
      //Serial.println(GPS.lastNMEA());   // this also sets the newNMEAreceived() flag to false

      if (!GPS.parse(GPS.lastNMEA())) {}  // this also sets the newNMEAreceived() flag to false
        //return;  // we can fail to parse a sentence in which case we should just wait for another
    }

    // approximately every 2 seconds or so, print out the current stats
    //if (millis() - timer > 2000) {
    //  timer = millis(); // reset the timer
    //  GPS_Serial_Print(GPS);

  //}
  //Serial.println(timeBetween);
  //delay(2000);
  }
}



/* ----------------------------------- START OF SETUP CODE ----------------------------------- */


void setup(void) {



  // HRM ----------------------------------------------------------------------
  pinMode(21,OUTPUT);         // pin that will blink to your heartbeat!
  // Configure Pin
  Serial.begin(115200);
  //pinMode(LED, OUTPUT);
  Serial.setTimeout(10000);

  //RTOS Setup -----------------------------------------------------------------------------------------------------------------------------------

    // Task to run forever
  xTaskCreatePinnedToCore(  // Use xTaskCreate() in vanilla FreeRTOS
    HRonSerial,              // Function to be called
    "Output HR to Serial Monitor",           // Name of task
    10000,                   // Stack size (bytes in ESP32, words in FreeRTOS)
    NULL,                   // Parameter to pass to function
    1,                      // Task priority (0 to configMAX_PRIORITIES - 1)
    &HRonSerialhandle,                   // Task handle
    1);               // Run on one core for demo purposes (ESP32 only)
    delay(500);
  
  

  // GPS ------------------------------------------------------------------------------------------------------------------------------------
  //vTaskStartScheduler();
  Serial.println("Adafruit GPS library basic parsing test!");


  // 9600 NMEA is the default baud rate for Adafruit MTK GPS's- some use 4800
  GPS.begin(9600);


  // uncomment this line to turn on RMC (recommended minimum) and GGA (fix data) including altitude
  GPS.sendCommand(PMTK_SET_NMEA_OUTPUT_RMCGGA);
  // uncomment this line to turn on only the "minimum recommended" data
  //  GPS.sendCommand(PMTK_SET_NMEA_OUTPUT_RMCONLY);
  // For parsing data, we don't suggest using anything but either RMC only or RMC+GGA since
  // the parser doesn't care about other sentences at this time

  // Set the update rate
  GPS.sendCommand(PMTK_SET_NMEA_UPDATE_1HZ);   // 1 Hz update rate
  // For the parsing code to work nicely and have time to sort thru the data, and
  // print it out we don't suggest using anything higher than 1 Hz

  // Request updates on antenna status, comment out to keep quiet
  GPS.sendCommand(PGCMD_ANTENNA);


  delay(1000);
  // Ask for firmware version
  // mySerial.println(PMTK_Q_RELEASE);


  // LCD ----------------------------------------------------------------------
  tft.init();
  tft.setRotation(0);
  tft.fillScreen(TFT_WHITE);




  // ACCELEROMETER setup --------------------------------------------------------------------------------------------------------------
  Wire.begin();
  Wire.beginTransmission(MPU_addr);
  Wire.write(0x6B);
  Wire.write(0);
  Wire.endTransmission(true);

  // Bluetooth Setup -----------------------------------------------------------------------------------------------------------------
  // Create the BLE Device. This is the name that will be advertised.
  BLEDevice::init("CoolProject");
  
  
  // Create the BLE Server
  pServer = BLEDevice::createServer();
  pServer->setCallbacks(new MyServerCallbacks());
  
  // Create the BLE Service
  BLEService *pService = pServer->createService(SERVICE_UUID);
  
  // Create a BLE Characteristic
  pTxCharacteristic = pService->createCharacteristic(
                    CHARACTERISTIC_UUID_TX,
                    BLECharacteristic::PROPERTY_NOTIFY
                  );
                     
  pTxCharacteristic->addDescriptor(new BLE2902());
  
  BLECharacteristic * pRxCharacteristic = pService->createCharacteristic(
                       CHARACTERISTIC_UUID_RX,
                      BLECharacteristic::PROPERTY_WRITE
                    );
  
  pRxCharacteristic->setCallbacks(new MyCallbacks());
  
  // Start the service
  pService->start();
  
  // Start advertising
  pServer->getAdvertising()->start();
  Serial.println("Waiting a client connection to notify...");
}



/* _______________________________________________________________________________________________________ */

/*                                             Start of Loop                                               */
/* _______________________________________________________________________________________________________ */

uint32_t timer = millis();
void loop() {

// GPS -------------------------------------------------------------------------------------------------------------------------------------------
  char c = GPS.read();
  // if you want to debug, this is a good time to do it!
  //if ((c) && (GPSECHO))
    //Serial.write(c);

  // if a sentence is received, we can check the checksum, parse it...
  if (GPS.newNMEAreceived()) {

    if (!GPS.parse(GPS.lastNMEA())){}   // this also sets the newNMEAreceived() flag to false
      //return;  // we can fail to parse a sentence in which case we should just wait for another
  }

  // approximately every 2 seconds or so, print out the current stats
   if (millis() - timer > 2000) {
    timer = millis(); // reset the timer


      velo = GPS.speed * 1.15078;
      
      currentLat = GPS.latitudeDegrees;
      currentLong = GPS.longitudeDegrees;

      if(previousLatAnchor != 0 && previousLongAnchor != 0){
          current_Distance_Traveled = sqrt(sq(69*(currentLat - previousLatAnchor)) + sq(54.6*(currentLong - previousLongAnchor)));
          if(current_Distance_Traveled > 0.01){
            Total_Distance_Traveled += current_Distance_Traveled;
            previousLatAnchor = currentLat;
            previousLongAnchor = currentLong;

            // this if statement handles mile split time
            if(Total_Distance_Traveled > currentMile){
              currentMile++;
              int currentTimeMinutes = GPS.hour*60 + GPS.minute;
              CurrentSplitTime_minutes = currentTimeMinutes - previousSplitTimeStart;

              previousSplitTimeStart = currentTimeMinutes;
              
            }

          }
          Serial.println(current_Distance_Traveled);
        
      }
      else{
        previousLatAnchor = currentLat;
        previousLongAnchor = currentLong;
        
        int currentTimeMinutes = 60*GPS.hour + GPS.minute;
        previousSplitTimeStart = currentTimeMinutes;

        startHour = GPS.hour;
        startSecond = GPS.seconds;
        startMinute = GPS.minute;

      }
      

      elapsedHours = GPS.hour-startHour;
      elapsedMinutes = GPS.minute-startMinute;
      elapsedSeconds = GPS.seconds-startSecond;

      if (elapsedSeconds<0){
        elapsedMinutes --;
        elapsedSeconds += 60;
      }
      if (elapsedMinutes<0){
        elapsedHours --;
        elapsedMinutes += 60;
      }

      sprintf(Current_elapsed_time, "%02d:%02d:%02d", elapsedHours, elapsedMinutes, elapsedSeconds);
      
      
      Serial.printf("Time: %s\n", Current_elapsed_time);    
      Serial.print("Speed: "); Serial.println(velo);
      Serial.print("Location: ");
      Serial.print(GPS.latitudeDegrees, 8);
      Serial.print(", ");
      Serial.println(GPS.longitudeDegrees, 8);
      Serial.print("Distance Traveled: "); Serial.println(Total_Distance_Traveled);
      Serial.print("Satellites: "); Serial.println((int)GPS.satellites);

      // CALORIES -------------------------------------------------------------------------------------------------------------------------------
      calories = caloriesBurned(weight, 3600*(elapsedHours) + 60*(elapsedMinutes) + elapsedSeconds, MET);
      Serial.print("Calories burned: "); Serial.println(calories);

      // ACCELEROMETER -----------------------------------------------------------------------------------------------------------------------
      Wire.beginTransmission(MPU_addr);
      Wire.write(0x3B);
      Wire.endTransmission(false);
      Wire.requestFrom(MPU_addr,14,true);
      AcX=Wire.read()<<8|Wire.read();
      AcY=Wire.read()<<8|Wire.read();
      AcZ=Wire.read()<<8|Wire.read();
      // int temp=Wire.read()<< 
      int xAng = map(AcX,minVal,maxVal,-90,90);
      int yAng = map(AcY,minVal,maxVal,-90,90);
      int zAng = map(AcZ,minVal,maxVal,-90,90);

      x= RAD_TO_DEG * (atan2(-yAng, -zAng)+PI);
      y= RAD_TO_DEG * (atan2(-xAng, -zAng)+PI);
      z= RAD_TO_DEG * (atan2(-yAng, -xAng)+PI);

      if(y>180) y=y-360;
      
      // Accounting for mount angle
      y += 20;

      Serial.print("AngleX= ");
      Serial.println(x);

      Serial.print("AngleY= ");
      Serial.println(y);

      Serial.print("AngleZ= ");
      Serial.println(z);

      Serial.println("------------------------");


      /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ BLUETOOTH CODE ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
      //do some sensor processing
      //s1 = Total_Distance_Traveled;
      //s2 = GPS.speed;
      //s3 = Current_elapsed_time;
      //s4 = calories;
      s5 = CurrentSplitTime_minutes;
      if(avBPM >30 && avBPM < 220){
        s6 = avBPM;
      }

      //you can use "Sr#:" to parse out each sensor data in the unity app
      //String sensor1 = "Sr1:";
      //String sensor2 = "Sr2:";
      String sensor3 = "Sr3:";
      //String sensor4 = "Sr4:";
      String sensor5 = "Sr5:";
      String sensor6 = "Sr6:";
      String sensor7 = "Sr7:";

      // concatenate strings with sensor values
      //sensor1 += s1;
      //sensor2 += s2;
      sensor3 += Current_elapsed_time;
      //sensor4 += s4;
      sensor5 += s5;
      sensor6 += s6;
      sensor7 += HR_sum;

      
      if (deviceConnected) {
            //convert strings to character arrays of 20 characters so we can send up to 20 bytes payload
            char a[20];
            char b[20];
            char c[20];
            char d[20];
            char e[20];
            char f[20];
            char q[20];
            char w[20];
            char z[20];

            sprintf(a, "Sr1:%.2f", Total_Distance_Traveled);
            sprintf(b, "Sr2:%.1f", GPS.speed);
            sprintf(d, "Sr4:%.1f", calories);
            sprintf(w, "Sr8:%.2f", HR_denom);
            sprintf(z, "Sr9:%.2f", HR_stored_avg);

            //sensor1.toCharArray(a,20);
            //sensor2.toCharArray(b,20);

            sensor3.toCharArray(c,20);
            //sensor4.toCharArray(d,20);
            sensor5.toCharArray(e,20);
            sensor6.toCharArray(f,20);
            sensor7.toCharArray(q,20);

            pTxCharacteristic->setValue((unsigned char*)a,20); //send 20 bytes
            pTxCharacteristic->notify();
        delay(30); // bluetooth stack will go into congestion, if too many packets are sent
            pTxCharacteristic->setValue((unsigned char*)b,20); //send 20 bytes
            pTxCharacteristic->notify();
        delay(30); // bluetooth stack will go into congestion, if too many packets are sent
            pTxCharacteristic->setValue((unsigned char*)c,20); //send 20 bytes
            pTxCharacteristic->notify();
        delay(30); // bluetooth stack will go into congestion, if too many packets are sent
            pTxCharacteristic->setValue((unsigned char*)d,20); //send 20 bytes
            pTxCharacteristic->notify();
        delay(30); // bluetooth stack will go into congestion, if too many packets are sent
            pTxCharacteristic->setValue((unsigned char*)e,20); //send 20 bytes
            pTxCharacteristic->notify();
        delay(30); // bluetooth stack will go into congestion, if too many packets are sent
            pTxCharacteristic->setValue((unsigned char*)f,20); //send 20 bytes
            pTxCharacteristic->notify();
        delay(30); // bluetooth stack will go into congestion, if too many packets are sent
            pTxCharacteristic->setValue((unsigned char*)q,20); //send 20 bytes
            pTxCharacteristic->notify();
        delay(30); // bluetooth stack will go into congestion, if too many packets are sent
        pTxCharacteristic->setValue((unsigned char*)w,20); //send 20 bytes
            pTxCharacteristic->notify();
        delay(30); // bluetooth stack will go into congestion, if too many packets are sent
        pTxCharacteristic->setValue((unsigned char*)z,20); //send 20 bytes
            pTxCharacteristic->notify();
        delay(30); // bluetooth stack will go into congestion, if too many packets are sent
      }

      // disconnecting
      if (!deviceConnected && oldDeviceConnected) {
          delay(500); // give the bluetooth stack the chance to get things ready
          pServer->startAdvertising(); // restart advertising
          Serial.println("start advertising");
          oldDeviceConnected = deviceConnected;
      }
      // connecting
      if (deviceConnected && !oldDeviceConnected) {
      // do stuff here on connecting
          oldDeviceConnected = deviceConnected;
      }
      /* END OF BLUETOOTH CODE */


      while (avBPM > 1000) 
        avBPM = avBPM / 10;



      /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ LCD CODE ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
      tft.setCursor(0, 0, 2);
      tft.fillScreen(TFT_WHITE);

      tft.setCursor(3,0);
      tft.setTextColor(TFT_BLACK,TFT_WHITE);  tft.setTextSize(4);  tft.setTextFont(2);
      
      if (velo<10) tft.printf("%.1f",velo);
      else tft.printf("%.0f",velo);

      tft.setCursor(tft.getCursorX(), 35);
      tft.setTextSize(1);  tft.setTextFont(2); tft.print(" MPH"); 
      tft.setCursor(tft.getCursorX(), 0);
      tft.setTextSize(4); tft.println("");

      tft.setTextColor(TFT_BLACK,TFT_WHITE);  tft.setTextSize(2);   tft.setTextFont(2); tft.setCursor(3, tft.getCursorY());
      tft.printf("%d",avBPM); 
      tft.setTextSize(1); tft.setCursor(tft.getCursorX(), tft.getCursorY()+10);
      tft.println(" BPM");

      tft.setTextColor(TFT_BLACK,TFT_WHITE); tft.setTextSize(2);   tft.setTextFont(2);  tft.setCursor(3, tft.getCursorY());
      tft.printf("%.2f",Total_Distance_Traveled);
      tft.setTextSize(1); tft.setCursor(tft.getCursorX(), tft.getCursorY()+10);
      tft.println(" mi");
      
      if (y != -135){ // gets angry sometimes
        tft.setTextColor(TFT_BLACK,TFT_WHITE); tft.setTextSize(2);   tft.setTextFont(2);  tft.setCursor(3, tft.getCursorY());
        tft.printf("%.1f",-y); 
        tft.setTextSize(1); tft.setCursor(tft.getCursorX(), tft.getCursorY()+10);
        tft.println(" deg");
      }


    }
  }